"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.orderbook = exports.ticker = exports.price = exports.Source = exports.Locale = exports.Interval = exports.WebSocketClient = exports.HttpClient = void 0;
const axios_1 = require("axios");
const WebSocket = require("ws");
class HttpClient {
    /**
     * Blockcc Api Client
     * @param apiKey
     * @param host
     */
    constructor(apiKey, host = "https://data.mifengcha.com") {
        this.host = host;
        this.apiKey = apiKey;
    }
    /**
     * Gets the ticker information for params
     * @param market
     * @param slug
     * @param symbol
     * @param currency
     * @param market_pair
     * @param page
     * @param size
     */
    async getTickers(market, slug, symbol, currency, market_pair, page, size) {
        return await get(this.host + "/api/v3/tickers", {
            market,
            slug,
            symbol,
            currency,
            market_pair,
            page,
            size,
            api_key: this.apiKey
        }, convertTickerData);
    }
    /**
     * Gets the price information for a given slug
     * @param slug
     * @param page
     * @param size
     */
    async getPrices(slug, page, size) {
        return await get(this.host + "/api/v3/price", {
            slug,
            page,
            size,
            api_key: this.apiKey
        }, convertPriceData);
    }
    /**
     * Gets the OrderBook information for a given desc
     * @param desc
     * @param limit
     */
    async getOrderBook(desc, limit) {
        return await get(this.host + "/api/v3/orderbook", {
            desc,
            limit,
            api_key: this.apiKey
        }, convertOrderBookData);
    }
    /**
     * Gets the Market information for a given slug
     * @param slug
     * @param page
     * @param size
     */
    async getMarkets(slug, page, size) {
        let url = "/api/v3/markets";
        if (slug != undefined) {
            url += "/" + slug;
        }
        return await get(this.host + url, {
            page,
            size,
            api_key: this.apiKey
        });
    }
    /**
     * Gets the Symbols information for a given slug
     * @param slug
     * @param details
     * @param page
     * @param size
     */
    async getSymbols(slug, details, page, size) {
        let url = "/api/v3/symbols";
        if (slug != undefined) {
            url += "/" + slug;
        }
        return await get(this.host + url, {
            details: details === true ? 1 : 0,
            page,
            size,
            api_key: this.apiKey
        });
    }
    /**
     * Gets the HistoryPrices information for given params
     * @param slug
     * @param start
     * @param end
     * @param interval
     */
    async getHistoryPrices(slug, start, end, interval) {
        return await get(this.host + "/api/v3/price/history?slug=" + slug, {
            start,
            end,
            interval,
            api_key: this.apiKey
        }, convertHistoryPriceData);
    }
    /**
     * Gets Trade information for a given desc
     * @param desc
     * @param limit
     */
    async getTrades(desc, limit) {
        return await get(this.host + "/api/v3/trades", {
            desc,
            limit,
            api_key: this.apiKey
        }, convertTradeData);
    }
    /**
     * Gets kline information for a given desc
     * @param desc
     * @param interval
     * @param end
     * @param start
     */
    async getKline(desc, interval, end, start) {
        return await get(this.host + "/api/v3/kline", {
            desc,
            interval,
            end,
            start,
            api_key: this.apiKey
        }, convertKlineData);
    }
    /**
     * Get the latest exchangeRate
     */
    async getExchangeRate() {
        return await get(this.host + "/api/v3/exchange_rate", {
            api_key: this.apiKey
        }, convertExchangeRateData);
    }
    /**
     * Get the latest briefs information
     * @param locale
     * @param page
     * @param size
     */
    async getBriefs(locale, page, size) {
        return await get(this.host + "/api/v3/briefs", {
            locale: locale,
            page: page,
            size: size,
            api_key: this.apiKey
        });
    }
    /**
     * Get the latest Announcements for markets
     * @param locale
     * @param market
     * @param page
     * @param size
     */
    async getAnnouncements(locale, market, page, size) {
        return await get(this.host + "/api/v3/announcements", {
            locale: locale,
            page: page,
            size: size,
            market: market,
            api_key: this.apiKey
        });
    }
    /**
     * Get articles information
     * @param locale
     * @param page
     * @param size
     */
    async getArticles(locale, page, size) {
        return await get(this.host + "/api/v3/articles", {
            locale: locale,
            page: page,
            size: size,
            api_key: this.apiKey
        });
    }
    /**
     * Get article for a given id
     * @param id
     */
    async getArticle(id) {
        return await get(this.host + "/api/v3/articles/" + id, {
            api_key: this.apiKey
        });
    }
    /**
     * Get social media for a given source
     * @param source
     * @param page
     * @param size
     */
    async getSocialMedia(source, page, size) {
        return await get(this.host + "/api/v3/social_media", {
            source: source,
            page: page,
            size: size,
            api_key: this.apiKey
        });
    }
}
exports.HttpClient = HttpClient;
class WebSocketClient {
    constructor(apiKey, host = "wss://data.mifengcha.com/ws/v3") {
        /**
         * subscribe topic
         * @param cb callback
         * @param params
         */
        this.subscribe = (cb, ...params) => {
            const wss = new WebSocket(this.host + "?api_key=" + this.apiKey);
            wss.on("open", () => {
                wss.send("{\"op\": \"subscribe\", \"args\": [" + params.toString() + "]}");
            });
            wss.onmessage = (msg) => {
                const { code: code, message: message, topic: topic, data: data } = JSON.parse(msg.data);
                cb({
                    code,
                    message,
                    topic,
                    data: parseData(topic, data)
                });
                return wss;
            };
        };
        this.host = host;
        this.apiKey = apiKey;
    }
}
exports.WebSocketClient = WebSocketClient;
async function get(url, params, convertData) {
    try {
        const response = await axios_1.default.get(url, { params });
        if (response.status !== 200 || !convertData) {
            return response.data;
        }
        return convertData(response.data);
    }
    catch (error) {
        return handleErrorResponse(error.response);
    }
}
function handleErrorResponse(error) {
    return {
        status: error.status,
        statusText: error.statusText,
        errorCode: error.data.c,
        errorMessage: error.data.m,
        error: error.data.e
    };
}
function convertOrderBookStream(orderBookData) {
    const asks = orderBookData.a.map(it => JSON.stringify(it));
    const bids = orderBookData.b.map(it => JSON.stringify(it));
    return {
        timestamp: orderBookData.T,
        asks: asks,
        bids: bids,
        marketPair: orderBookData.m
    };
}
function convertOnePrice(priceData) {
    return {
        symbolName: priceData.s,
        symbol: priceData.S,
        timestamp: priceData.T,
        priceUsd: priceData.u,
        priceBtc: priceData.b,
        volume: priceData.a,
        volumeUsd: priceData.v,
        reportedVolume: priceData.ra,
        reportedVolumeUsd: priceData.rv,
        marketCapUsd: priceData.m,
        priceChange1d: priceData.c,
        highPrice1d: priceData.h,
        lowPrice1d: priceData.l,
        priceChange1w: priceData.cw,
        highPrice1w: priceData.hw,
        lowPrice1w: priceData.lw,
        priceChange1m: priceData.cm,
        highPrice1m: priceData.hm,
        lowPrice1m: priceData.lm,
        highPriceAll: priceData.ha,
        lowPriceAll: priceData.la
    };
}
function convertOneTicker(tickerData) {
    return {
        timestamp: tickerData.T,
        marketPair: tickerData.m,
        openPrice: tickerData.o,
        lastPrice: tickerData.c,
        lowPrice: tickerData.l,
        highPrice: tickerData.h,
        askPrice: tickerData.a,
        askAmount: tickerData.A,
        bidPrice: tickerData.b,
        changeDaily: tickerData.C,
        baseVolume: tickerData.bv,
        quoteVolume: tickerData.qv,
        usdRate: tickerData.r,
        purity: tickerData.p,
        spread: tickerData.s
    };
}
function convertOrderBookData(data) {
    return {
        timestamp: data.T,
        asks: data.a,
        bids: data.b,
        marketPair: data.m
    };
}
function convertPriceData(data) {
    return data.map(it => convertOnePrice(it));
}
function convertHistoryPriceData(data) {
    return data.map(it => {
        return {
            timestamp: it.T,
            priceUsd: it.u,
            priceBtc: it.b,
            volume: it.a,
            volumeUsd: it.v,
            marketCapUsd: it.m
        };
    });
}
function convertExchangeRateData(data) {
    return data.map(it => {
        return {
            currency: it.c,
            rate: it.r
        };
    });
}
function convertTickerData(data) {
    return data.map(it => convertOneTicker(it));
}
function convertTradeData(data) {
    return data.map(it => {
        return {
            timestamp: it.T,
            marketPair: it.m,
            tradeType: it.s,
            price: it.p,
            volume: it.v
        };
    });
}
function convertKlineData(data) {
    return data.map(it => {
        return {
            timestamp: it.T,
            open: it.o,
            high: it.h,
            low: it.l,
            close: it.c,
            volume: it.v
        };
    });
}
function parseData(topic, data) {
    if (topic === undefined) {
        return undefined;
    }
    if (topic.indexOf("ticker") !== -1) {
        return convertOneTicker(data);
    }
    else if (topic.indexOf("orderbook") !== -1) {
        return convertOrderBookStream(data);
    }
    else if (topic.indexOf("price") !== -1) {
        return convertOnePrice(data);
    }
    else {
        return undefined;
    }
}
class Interval {
}
exports.Interval = Interval;
Interval.ONE_MIN = "1m";
Interval.FIVE_MIN = "5m";
Interval.FIFTEEN_MIN = "15m";
Interval.THIRTY_MIN = "30m";
Interval.ONE_HOUR = "1h";
Interval.TWO_HOUR = "2h";
Interval.SIX_HOUR = "6h";
Interval.TWELVE_HOUR = "12h";
Interval.ONE_DAY = "1d";
Interval.TWO_DAY = "2d";
class Locale {
}
exports.Locale = Locale;
Locale.zh_CN = "zh_CN";
Locale.en_US = "en_US";
Locale.ko_KR = "ko_KR";
class Source {
}
exports.Source = Source;
Source.WEIBO = "WEIBO";
Source.TWITTER = "TWITTER";
function price(coinId) {
    return "\"price:" + coinId + "\"";
}
exports.price = price;
function ticker(desc) {
    return "\"ticker:" + desc + "\"";
}
exports.ticker = ticker;
function orderbook(desc) {
    return "\"orderbook:" + desc + "\"";
}
exports.orderbook = orderbook;
//# sourceMappingURL=blockcc-api.js.map